# -*- coding: utf-8 -*-
#Common
import os

# file_cleanup
from django.core.files.storage import default_storage
from django.db.models import FileField

# OverwriteStorage
from django.conf import settings
 
def file_cleanup(sender, **kwargs):
    """
    File cleanup callback used to emulate the old delete
    behavior using signals. Initially django deleted linked
    files when an object containing a File/ImageField was deleted.
 
    Usage:
    >>> from django.db.models.signals import post_delete
    >>> post_delete.connect(file_cleanup, sender=MyModel, dispatch_uid="mymodel.file_cleanup")
    """
    for fieldname in sender._meta.get_all_field_names():
        try:
            field = sender._meta.get_field(fieldname)
        except:
            field = None
        if field and isinstance(field, FileField):
            inst = kwargs['instance']
            f = getattr(inst, fieldname)
            m = inst.__class__._default_manager
            if hasattr(f, 'path') and os.path.exists(f.path)\
            and not m.filter(**{'%s__exact' % fieldname: getattr(inst, fieldname)})\
            .exclude(pk=inst._get_pk_val()):
                try:
                    default_storage.delete(f.path)
                except:
                    pass


from django.core.files.storage import FileSystemStorage
class OverwriteStorage(FileSystemStorage):
    def get_available_name(self, name):
        """Returns a filename that's free on the target storage system, and
        available for new content to be written to.
 
        Found at http://djangosnippets.org/snippets/976/
 
        This file storage solves overwrite on upload problem. Another
        proposed solution was to override the save method on the model
        like so (from https://code.djangoproject.com/ticket/11663):
 
        def save(self, *args, **kwargs):
            try:
                this = MyModelName.objects.get(id=self.id)
                if this.MyImageFieldName != self.MyImageFieldName:
                    this.MyImageFieldName.delete()
            except: pass
            super(MyModelName, self).save(*args, **kwargs)
        """
        # If the filename already exists, remove it as if it was a true file system
        if self.exists(name):
            os.remove(os.path.join(settings.MEDIA_ROOT, name))
        return name


#-*- coding: utf-8 -*-
import os
from django.http import HttpResponse
from django.template import RequestContext, loader
from django.core.urlresolvers import reverse
from django.views.decorators.csrf import csrf_exempt
from django.templatetags.static import static

from django.contrib.staticfiles import finders
from django.template import engines

@csrf_exempt
def render_static_file(request, static_file=None):
    if static_file is None:
        query_dict = request.POST
        static_file = query_dict['static_file']

    path_static_file = finders.find(static_file)
    f = open(path_static_file, 'r')
    data = f.read()

    t = engines['django'].from_string(data)
    # t = loader.get_template_from_string(data)
    c = RequestContext(request)
    return HttpResponse(t.render(c))

@csrf_exempt
def url_reverse(request, view=None, args=None):
    if view is None and args is None:
        query_dict = request.POST
        view = query_dict['view']
        args = query_dict['args']
    arg_list = args.split(',')
    return HttpResponse(reverse(view, args=arg_list))


# -*- coding: utf-8 -*-
import sys

import logging
log = logging.getLogger(__name__)
handler = logging.StreamHandler(stream=sys.stdout)
handler.setLevel(logging.DEBUG)
log.addHandler(handler)
log.setLevel(logging.DEBUG)

from arclib.django.object import ArcReturnJSONObject




# -*- coding: utf-8 -*-
import os
import sys

import logging
log = logging.getLogger(__name__)
handler = logging.StreamHandler(stream=sys.stdout)
handler.setLevel(logging.DEBUG)
log.addHandler(handler)
log.setLevel(logging.DEBUG)

import json
import timeit
import requests

from arclib.django.object import ArcReturnJSONObject

class TimeitHttpClient:
    def __init__(self, title, url, method='post', params=None, files=None, repeat_number=1, loop=False, loop_count=1, handler=None):
        self.title = title
        self.url = url
        self.method = method
        self.params = params
        self.files = files
        self.repeat_number = repeat_number
        self.loop = loop
        self.loop_count = loop_count
        self.result_handler = handler

    # result_handler는 String result값을 받아 처리하는 함수
    # result(String)과 count(Integer)를 인자로 가짐
    def set_result_handler(self, handler):
        self.result_handler = handler

    def process(self, count):
        def request_http():
            if self.method == 'post':
                r = requests.post(self.url, data=self.params, files=self.files)
            else:
                r = requests.get(self.url, data=self.params)

            result = r.text
            self.result_handler(result, self.title, count)
        try:
            t = timeit.Timer(lambda : request_http())
            total_time = t.timeit(number=self.repeat_number)
            log.debug(u'  %s execution average time : %.02f (%s repeat)\n' %
                      (self.title, (total_time / self.repeat_number), self.repeat_number))
        except Exception as e:
            log.debug('Exception args : %s' % (e.args))
            log.debug('Exception : %s' % (e))
            raise Exception

    def execute(self):
        count = 0
        try:
            log.debug(u'\n-- %s measure execution time Start --\n' % (self.title))
            while True:
                self.process(count)
                count += 1
                if self.loop is not True and count == self.loop_count:
                    break
        except KeyboardInterrupt as e:
            log.debug(u'   !! User keyboard interrupt !!')
        finally:
            log.debug(u'\n-- %s measure execution time End --\n' % (self.title))


# -*- coding: utf-8 -*-
from django.db.models.signals import post_init
 
def track_data(*fields):
    """
    Tracks property changes on a model instance.
    
    The changed list of properties is refreshed on model initialization
    and save.
    
    >>> @track_data('name')
    >>> class Post(models.Model):
    >>>     name = models.CharField(...)
    >>> 
    >>>     @classmethod
    >>>     def post_save(cls, sender, instance, created, **kwargs):
    >>>         if instance.has_changed('name'):
    >>>             print "Hooray!"
    """
    
    UNSAVED = dict()
 
    def _store(self):
        "Updates a local copy of attributes values"
        if self.id:
            self.__data = dict((f, getattr(self, f)) for f in fields)
        else:
            self.__data = UNSAVED
 
    def inner(cls):
        # contains a local copy of the previous values of attributes
        cls.__data = {}
 
        def has_changed(self, field):
            "Returns ``True`` if ``field`` has changed since initialization."
            if self.__data is UNSAVED:
                return False
            return self.__data.get(field) != getattr(self, field)
        cls.has_changed = has_changed
 
        def old_value(self, field):
            "Returns the previous value of ``field``"
            return self.__data.get(field)
        cls.old_value = old_value
 
        def whats_changed(self):
            "Returns a list of changed attributes."
            changed = {}
            if self.__data is UNSAVED:
                return changed
            for k, v in self.__data.iteritems():
                if v != getattr(self, k):
                    changed[k] = v
            return changed
        cls.whats_changed = whats_changed
 
        # Ensure we are updating local attributes on model init
        def _post_init(sender, instance, **kwargs):
            _store(instance)
        post_init.connect(_post_init, sender=cls, weak=False)
 
        # Ensure we are updating local attributes on model save
        def save(self, *args, **kwargs):
            save._original(self, *args, **kwargs)
            _store(self)
        save._original = cls.save
        cls.save = save
        return cls
    return inner